define bi_wget::wget (
  $download_url = $title,
  $output_document = undef,
  $quiet = true,
  $logfile = undef,
  $append_output = undef,
  $input_file = undef,
  $debug = false,
  $verbose = false,
  $no_verbose = false,
  $force_html = false,
  $base = undef,
  $bind_address = undef,
  $tries = 20,
  $timeout = 300,
  $no_clobber = false,
  $backups = undef,
  $mode = undef,
  $owner = undef,
  $group = undef,
  $refreshonly = false,
) {
    require bi_coreutils
    require bi_wget
  
    if !$download_url { fail("download_url must be defined") }
    if !$output_document { fail("output_document must be defined") }
    
    if $bind_address {
        $arg_bind_address = is_string($bind_address) ? {
            true => "--bind-address=$bind_address",
            default => undef,
        }
    }
  
    if $output_document {
        $arg_output_document = is_string($output_document) ? {
            true => "--output-document=$output_document",
            default => undef,
        }
    }
  
    if $append_output {
        $arg_append_output = is_string($append_output) ? {
            true => "--append-output=$append_output",
            default => undef,
        }
    }
  
    if $logfile {
        $arg_logfile = is_string($logfile) ? {
            true => "--output-file=$logfile",
            default => undef,
        }
    }
    
    if $input_file {
        $arg_input_file = is_string($input_file) ? {
            true => "--input-file=$input_file",
            default => undef,
        }
    }
    
    if $base {
        $arg_base = is_string($base) ? {
            true => "--base=$base",
            default => undef,
        }
    }
    
    if $tries {
        $arg_tries = is_integer($tries) ? {
            true => "--tries=$tries",
            default => '--tries=20',
        }
    }
    
    if $backups {
        $arg_backups = is_integer($backups) ? {
            true => "--backups=$backups",
            default => undef,
        }
    }
    
    if is_bool($no_clobber) {
        $arg_no_clobber = $no_clobber ? {
            true => '--no-clobber',
            default => undef,
        }
    }
    
    if is_bool($no_clobber) {
        $creates = $no_clobber ? {
            true => $output_document,
            default => undef,
        }
    }
    
    if is_bool($no_verbose) {
        $arg_no_verbose = $no_verbose ? {
            true => '--no-verbose',
            default => undef,
        }
    }
    
    if is_bool($verbose) {
        $arg_verbose = $verbose ? {
            true => '--verbose',
            default => undef,
        }
    }
    
    if is_bool($quiet) {
        $arg_quiet = $quiet ? {
            true => '--quiet',
            default => undef,
        }
    }
    
    if is_bool($debug) {
        $arg_debug = $debug ? {
            true => '--debug',
            default => undef,
        }
    }
    
    $args = join(delete_undef_values([ 
        $arg_quiet, 
        $arg_output_document,
        $arg_logfile,
        $arg_debug,
        $arg_append_output,
        $arg_no_verbose,
        $arg_verbose,
        $arg_input_file,
        $arg_base,
        $arg_bind_address,
        $arg_tries,
        $arg_no_clobber,
        $arg_backups,
    ]), " ")
    
    exec { "$bi_wget::exe_path $args $download_url":
        creates => $creates,
        timeout => $timeout,
        refreshonly => $refreshonly,
    }
    
    if $mode and is_numeric($mode) {
        bi_coreutils::chmod { $output_document:
            mode => $mode,
            require => Exec["$bi_wget::exe_path $args $download_url"],
            refreshonly => $refreshonly,
        }
    }
    
    if ($owner and is_string($owner)) or ($group and is_string($group)) {
        bi_coreutils::chown { $output_document:
            owner => $owner,
            group => $group,
            require => Exec["$bi_wget::exe_path $args $download_url"],
            refreshonly => $refreshonly,
        }
    }
  
}